"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs");
const chai = require("chai");
const chaiAsPromised = require("chai-as-promised");
const can_deploy_1 = require("./can-deploy");
const logger_1 = require("./logger");
const broker_mock_1 = require("../test/integration/broker-mock");
const rimraf = require("rimraf");
const mkdirp = require("mkdirp");
const expect = chai.expect;
chai.use(chaiAsPromised);
describe('CanDeploy Spec', () => {
    const PORT = Math.floor(Math.random() * 999) + 9000;
    let server;
    let absolutePath;
    let relativePath;
    before(() => broker_mock_1.default(PORT).then(s => {
        logger_1.default.debug(`Pact Broker Mock listening on port: ${PORT}`);
        server = s;
    }));
    after(() => server.close());
    beforeEach(() => {
        relativePath = `.tmp/${Math.floor(Math.random() * 1000)}`;
        absolutePath = path.resolve(__dirname, '..', relativePath);
        mkdirp.sync(absolutePath);
    });
    afterEach(() => {
        if (fs.existsSync(absolutePath)) {
            rimraf.sync(absolutePath);
        }
    });
    describe('convertForSpawnBinary helper function', () => {
        it('produces an array of SpawnArguments', () => {
            const value = { pactBroker: 'some broker', pacticipants: [] };
            const result = can_deploy_1.CanDeploy.convertForSpawnBinary(value);
            expect(result).to.be.an('array');
            expect(result.length).to.be.equal(1);
            expect(result).to.be.deep.equal([{ pactBroker: 'some broker' }]);
        });
        it('has version and participant in the right order', () => {
            const result = can_deploy_1.CanDeploy.convertForSpawnBinary({
                pacticipants: [{ version: 'v2', name: 'one' }],
                pactBroker: 'some broker',
                pactBrokerUsername: 'username',
                pactBrokerPassword: 'password',
            });
            expect(result).to.eql([
                {
                    pactBroker: 'some broker',
                    pactBrokerUsername: 'username',
                    pactBrokerPassword: 'password',
                },
                { name: 'one' },
                { version: 'v2' },
            ]);
        });
        it('has latest tag and participant in the right order', () => {
            const result = can_deploy_1.CanDeploy.convertForSpawnBinary({
                pacticipants: [{ name: 'two', latest: 'SOME_TAG' }],
                pactBroker: 'some broker',
            });
            expect(result).to.eql([
                {
                    pactBroker: 'some broker',
                },
                { name: 'two' },
                { latest: 'SOME_TAG' },
            ]);
        });
        it("understands 'true' for latest", () => {
            const result = can_deploy_1.CanDeploy.convertForSpawnBinary({
                pacticipants: [{ name: 'two', latest: true }],
                pactBroker: 'some broker',
            });
            expect(result).to.eql([
                {
                    pactBroker: 'some broker',
                },
                { name: 'two' },
                { latest: 'PACT_NODE_NO_VALUE' },
            ]);
        });
    });
    context('when invalid options are set', () => {
        it('should fail with an Error when not given pactBroker', () => {
            expect(() => can_deploy_1.default({})).to.throw(Error);
        });
        it('should fail with an error when there are no paticipants', () => {
            expect(() => can_deploy_1.default({
                pactBroker: 'http://localhost',
                pacticipants: [],
            })).to.throw(Error);
        });
    });
    context('when valid options are set', () => {
        it('should return a CanDeploy object when given the correct arguments', () => {
            const c = can_deploy_1.default({
                pactBroker: 'http://localhost',
                pacticipants: [{ name: 'two', version: '2' }],
            });
            expect(c).to.be.ok;
            expect(c.canDeploy).to.be.a('function');
        });
    });
    context('candeploy function', () => {
        it('should return success with a table result deployable true', done => {
            const opts = {
                pactBroker: `http://localhost:${PORT}`,
                pacticipants: [{ name: 'Foo', version: '4' }],
            };
            const ding = can_deploy_1.default(opts);
            ding.canDeploy().then(results => {
                expect(results).not.to.be.null;
                done();
            });
        });
        context('with latest true', () => {
            it('should return success with a table result deployable true', done => {
                const opts = {
                    pactBroker: `http://localhost:${PORT}`,
                    pacticipants: [{ name: 'Foo', latest: true }],
                };
                const ding = can_deploy_1.default(opts);
                ding.canDeploy().then(results => {
                    expect(results).not.to.be.null;
                    done();
                });
            });
            it('should throw an error with a table result deployable false', () => {
                const opts = {
                    pactBroker: `http://localhost:${PORT}`,
                    pacticipants: [{ name: 'FooFail', latest: true }],
                };
                const ding = can_deploy_1.default(opts);
                return ding
                    .canDeploy()
                    .then(() => expect.fail())
                    .catch(message => expect(message).not.be.null);
            });
        });
        context('with latest a string', () => {
            it('should return success with a table result deployable true', done => {
                const opts = {
                    pactBroker: `http://localhost:${PORT}`,
                    pacticipants: [{ name: 'Foo', latest: 'tag' }],
                };
                const ding = can_deploy_1.default(opts);
                ding.canDeploy().then(results => {
                    expect(results).not.to.be.null;
                    done();
                });
            });
            it('should throw an error with a table result deployable false', () => {
                const opts = {
                    pactBroker: `http://localhost:${PORT}`,
                    pacticipants: [{ name: 'FooFail', latest: 'tag' }],
                };
                const ding = can_deploy_1.default(opts);
                return ding
                    .canDeploy()
                    .then(() => expect.fail())
                    .catch(message => expect(message).not.be.null);
            });
        });
        context('with latest a string, and a to', () => {
            it('should return success with a table result deployable true', done => {
                const opts = {
                    pactBroker: `http://localhost:${PORT}`,
                    pacticipants: [{ name: 'Foo', latest: 'tag' }],
                    to: 'prod',
                };
                const ding = can_deploy_1.default(opts);
                ding.canDeploy().then(results => {
                    expect(results).not.to.be.null;
                    done();
                });
            });
            it('should throw an error with a table result deployable false', () => {
                const opts = {
                    pactBroker: `http://localhost:${PORT}`,
                    pacticipants: [{ name: 'FooFail', latest: 'tag' }],
                    to: 'prod',
                };
                const ding = can_deploy_1.default(opts);
                return ding
                    .canDeploy()
                    .then(() => expect.fail())
                    .catch(message => expect(message).not.be.null);
            });
        });
        it('should throw an error with a table result deployable false', () => {
            const opts = {
                pactBroker: `http://localhost:${PORT}`,
                pacticipants: [{ name: 'FooFail', version: '4' }],
            };
            const ding = can_deploy_1.default(opts);
            return ding
                .canDeploy()
                .then(() => expect.fail())
                .catch(message => expect(message).not.be.null);
        });
        it('should return success with a json result deployable true', done => {
            const opts = {
                pactBroker: `http://localhost:${PORT}`,
                pacticipants: [{ name: 'Foo', version: '4' }],
                output: 'json',
            };
            const ding = can_deploy_1.default(opts);
            ding.canDeploy().then(results => {
                expect(results).not.to.be.null;
                done();
            });
        });
        it('should throw an error with a json result deployable false', () => {
            const opts = {
                pactBroker: `http://localhost:${PORT}`,
                pacticipants: [{ name: 'FooFail', version: '4' }],
                output: 'json',
            };
            const ding = can_deploy_1.default(opts);
            return ding
                .canDeploy()
                .then(() => expect.fail())
                .catch(message => expect(message).not.be.null);
        });
    });
});
//# sourceMappingURL=can-deploy.spec.js.map