"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai = require("chai");
const path = require("path");
const chaiAsPromised = require("chai-as-promised");
const fs = require("fs");
const message_1 = require("./message");
const mkdirp = require("mkdirp");
const rimraf = require("rimraf");
const expect = chai.expect;
chai.use(chaiAsPromised);
describe('Message Spec', () => {
    const validJSON = `{ "description": "a test mesage", "content": { "name": "Mary" } }`;
    let absolutePath;
    let relativePath;
    beforeEach(() => {
        relativePath = `.tmp/${Math.floor(Math.random() * 1000)}`;
        absolutePath = path.resolve(__dirname, '..', relativePath);
        mkdirp.sync(absolutePath);
    });
    afterEach(() => {
        if (fs.existsSync(absolutePath)) {
            rimraf.sync(absolutePath);
        }
    });
    context('when invalid options are set', () => {
        it('should throw an Error when not given any message content', () => {
            expect(() => message_1.default({
                consumer: 'a-consumer',
                dir: absolutePath,
            })).to.throw(Error);
        });
        it('should throw an Error when not given a consumer', () => {
            expect(() => message_1.default({
                provider: 'a-provider',
                dir: absolutePath,
                content: validJSON,
            })).to.throw(Error);
        });
        it('should throw an Error when not given a provider', () => {
            expect(() => message_1.default({
                consumer: 'a-provider',
                dir: absolutePath,
                content: validJSON,
            })).to.throw(Error);
        });
        it('should throw an Error when given an invalid JSON document', () => {
            expect(() => message_1.default({
                consumer: 'some-consumer',
                provider: 'a-provider',
                dir: absolutePath,
                content: `{ "unparseable" }`,
            })).to.throw(Error);
        });
        it('should throw an Error when not given a pact dir', () => {
            expect(() => message_1.default({
                consumer: 'a-consumer',
                content: validJSON,
            })).to.throw(Error);
        });
    });
    context('when valid options are set', () => {
        it('should return a message object when given the correct arguments', () => {
            const message = message_1.default({
                consumer: 'some-consumer',
                provider: 'a-provider',
                dir: absolutePath,
                content: validJSON,
            });
            expect(message).to.be.a('object');
            expect(message).to.respondTo('createMessage');
        });
        it('should return a promise when calling createMessage', () => {
            const promise = message_1.default({
                consumer: 'some-consumer',
                provider: 'a-provider',
                dir: absolutePath,
                content: validJSON,
            }).createMessage();
            expect(promise).to.ok;
            return expect(promise).to.eventually.be.fulfilled;
        });
        it("should create a new directory if the directory specified doesn't exist yet", () => {
            const dir = path.resolve(absolutePath, 'create');
            return message_1.default({
                consumer: 'some-consumer',
                provider: 'a-provider',
                dir: dir,
                content: validJSON,
            })
                .createMessage()
                .then(() => expect(fs.existsSync(dir)).to.be.true);
        });
        it('should return an absolute path when a relative one is given', () => {
            const dir = path.join(relativePath, 'create');
            expect(message_1.default({
                consumer: 'some-consumer',
                provider: 'a-provider',
                dir: dir,
                content: validJSON,
            }).options.dir).to.equal(path.resolve(__dirname, '..', dir));
        });
        it('should create a new directory with a relative path', () => {
            const dir = path.join(relativePath, 'create');
            return message_1.default({
                consumer: 'some-consumer',
                provider: 'a-provider',
                dir: dir,
                content: validJSON,
            })
                .createMessage()
                .then(() => expect(fs.existsSync(path.resolve(__dirname, '..', dir))).to.be
                .true);
        });
    });
});
//# sourceMappingURL=message.spec.js.map