"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const q = require("q");
const path = require("path");
const fs = require("fs");
const logger_1 = require("./logger");
const spawn_1 = require("./spawn");
const spawn_2 = require("./spawn");
const util_1 = require("util");
const pact_standalone_1 = require("./pact-standalone");
const checkTypes = require('check-types');
class Publisher {
    constructor(options) {
        this.__argMapping = {
            pactFilesOrDirs: spawn_2.DEFAULT_ARG,
            pactBroker: '--broker-base-url',
            pactBrokerUsername: '--broker-username',
            pactBrokerPassword: '--broker-password',
            pactBrokerToken: '--broker-token',
            tags: '--tag',
            consumerVersion: '--consumer-app-version',
            verbose: '--verbose',
        };
        options = options || {};
        options.tags = options.tags || [];
        options.timeout = options.timeout || 60000;
        checkTypes.assert.nonEmptyString(options.pactBroker, 'Must provide the pactBroker argument');
        checkTypes.assert.nonEmptyString(options.consumerVersion, 'Must provide the consumerVersion argument');
        checkTypes.assert.arrayLike(options.pactFilesOrDirs, 'Must provide the pactFilesOrDirs argument');
        checkTypes.assert.nonEmptyArray(options.pactFilesOrDirs, 'Must provide the pactFilesOrDirs argument with an array');
        if (options.pactFilesOrDirs) {
            checkTypes.assert.array.of.string(options.pactFilesOrDirs);
            options.pactFilesOrDirs = options.pactFilesOrDirs.map(v => {
                const newPath = path.resolve(v);
                if (!fs.existsSync(newPath)) {
                    throw new Error(`Path '${v}' given in pactFilesOrDirs does not exists.`);
                }
                return newPath;
            });
        }
        if (options.pactBroker) {
            checkTypes.assert.string(options.pactBroker);
        }
        if (options.pactBrokerUsername) {
            checkTypes.assert.string(options.pactBrokerUsername);
        }
        if (options.pactBrokerPassword) {
            checkTypes.assert.string(options.pactBrokerPassword);
        }
        if (options.verbose === undefined && logger_1.verboseIsImplied()) {
            options.verbose = true;
        }
        if ((options.pactBrokerUsername && !options.pactBrokerPassword) ||
            (options.pactBrokerPassword && !options.pactBrokerUsername)) {
            throw new Error('Must provide both Pact Broker username and password. None needed if authentication on Broker is disabled.');
        }
        if (options.pactBrokerToken &&
            (options.pactBrokerUsername || options.pactBrokerPassword)) {
            throw new Error('Must provide pactBrokerToken or pactBrokerUsername/pactBrokerPassword but not both.');
        }
        this.options = options;
    }
    publish() {
        logger_1.default.info(`Publishing pacts to broker at: ${this.options.pactBroker}`);
        const deferred = q.defer();
        const instance = spawn_1.default.spawnBinary(pact_standalone_1.default.brokerPath, [{ cliVerb: 'publish' }, this.options], this.__argMapping);
        const output = [];
        instance.stdout.on('data', l => output.push(l));
        instance.stderr.on('data', l => output.push(l));
        instance.once('close', code => {
            const o = output.join('\n');
            const pactUrls = /^https?:\/\/.*\/pacts\/.*$/gim.exec(o);
            if (code !== 0 || !pactUrls) {
                logger_1.default.error(`Could not publish pact:\n${o}`);
                return deferred.reject(new Error(o));
            }
            logger_1.default.info(o);
            return deferred.resolve(pactUrls);
        });
        return deferred.promise.timeout(this.options.timeout, `Timeout waiting for verification process to complete (PID: ${instance.pid})`);
    }
}
Publisher.create = util_1.deprecate((options) => new Publisher(options), 'Create function will be removed in future release, please use the default export function or use `new Publisher()`');
exports.Publisher = Publisher;
exports.default = (options) => new Publisher(options);
//# sourceMappingURL=publisher.js.map