"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const server_1 = require("./server");
const chai = require("chai");
const chaiAsPromised = require("chai-as-promised");
const fs = require("fs");
const util = require("util");
const path = require("path");
const q = require("q");
const _ = require("underscore");
const mkdirp = require("mkdirp");
const rimraf = require("rimraf");
chai.use(chaiAsPromised);
const expect = chai.expect;
const rm = util.promisify(rimraf);
describe('Server Spec', () => {
    let server;
    const monkeypatchFile = path.resolve(__dirname, '../test/monkeypatch.rb');
    let absolutePath;
    let relativePath;
    beforeEach(() => {
        relativePath = `.tmp/${Math.floor(Math.random() * 1000)}`;
        absolutePath = path.resolve(__dirname, '..', relativePath);
        mkdirp.sync(absolutePath);
    });
    afterEach(() => __awaiter(this, void 0, void 0, function* () {
        if (server) {
            yield server.delete();
        }
        if (fs.existsSync(absolutePath)) {
            yield rm(absolutePath);
        }
    }));
    const relativeSSLCertPath = 'test/ssl/server.crt';
    const absoluteSSLCertPath = path.resolve(__dirname, '..', relativeSSLCertPath);
    const relativeSSLKeyPath = 'test/ssl/server.key';
    const absoluteSSLKeyPath = path.resolve(__dirname, '..', relativeSSLKeyPath);
    describe('Start server', () => {
        context('when no options are set', () => {
            it('should start correctly with defaults', () => {
                server = server_1.default();
                return expect(server.start()).to.eventually.be.fulfilled;
            });
        });
        context('when invalid options are set', () => {
            it('should fail if custom ssl certs do not exist', () => {
                expect(() => server_1.default({
                    ssl: true,
                    sslcert: 'does/not/exist',
                    sslkey: absoluteSSLKeyPath,
                })).to.throw(Error);
            });
            it('should fail if custom ssl keys do not exist', () => {
                expect(() => server_1.default({
                    ssl: true,
                    sslcert: absoluteSSLCertPath,
                    sslkey: 'does/not/exist',
                })).to.throw(Error);
            });
            it("should fail if custom ssl cert is set, but ssl key isn't", () => {
                expect(() => server_1.default({
                    ssl: true,
                    sslcert: absoluteSSLCertPath,
                })).to.throw(Error);
            });
            it("should fail if custom ssl key is set, but ssl cert isn't", () => {
                expect(() => server_1.default({
                    ssl: true,
                    sslkey: absoluteSSLKeyPath,
                })).to.throw(Error);
            });
        });
        context('when valid options are set', () => {
            it('should start correctly when instance is delayed', () => {
                server = server_1.default();
                const waitForServerUp = server['__waitForServiceUp'].bind(server);
                return q
                    .allSettled([
                    waitForServerUp(server.options),
                    q.delay(5000).then(() => server.start()),
                ])
                    .then(results => expect(_.reduce(results, (m, r) => m && r.state === 'fulfilled'))
                    .to.be.true);
            });
            it('should start correctly with ssl', () => {
                server = server_1.default({ ssl: true });
                expect(server.options.ssl).to.equal(true);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with custom ssl cert/key', () => {
                server = server_1.default({
                    ssl: true,
                    sslcert: absoluteSSLCertPath,
                    sslkey: absoluteSSLKeyPath,
                });
                expect(server.options.ssl).to.equal(true);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with custom ssl cert/key but without specifying ssl flag', () => {
                server = server_1.default({
                    sslcert: absoluteSSLCertPath,
                    sslkey: absoluteSSLKeyPath,
                });
                expect(server.options.ssl).to.equal(true);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with cors', () => {
                server = server_1.default({ cors: true });
                expect(server.options.cors).to.equal(true);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with port', () => {
                const port = Math.floor(Math.random() * 999) + 9000;
                server = server_1.default({ port: port });
                expect(server.options.port).to.equal(port);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with host', () => {
                const host = 'localhost';
                server = server_1.default({ host: host });
                expect(server.options.host).to.equal(host);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with spec version 1', () => {
                server = server_1.default({ spec: 1 });
                expect(server.options.spec).to.equal(1);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with spec version 2', () => {
                server = server_1.default({ spec: 2 });
                expect(server.options.spec).to.equal(2);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with log', () => {
                const logPath = path.resolve(absolutePath, 'log.txt');
                server = server_1.default({ log: logPath });
                expect(server.options.log).to.equal(logPath);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with consumer name', () => {
                const consumerName = 'cName';
                server = server_1.default({ consumer: consumerName });
                expect(server.options.consumer).to.equal(consumerName);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with provider name', () => {
                const providerName = 'pName';
                server = server_1.default({ provider: providerName });
                expect(server.options.provider).to.equal(providerName);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with monkeypatch', () => {
                const s = server_1.default({ monkeypatch: monkeypatchFile });
                expect(s.options.monkeypatch).to.equal(monkeypatchFile);
                return expect(s.start()).to.eventually.be.fulfilled;
            });
            context('Paths', () => {
                it('should start correctly with dir, absolute path', () => {
                    server = server_1.default({ dir: relativePath });
                    expect(server.options.dir).to.equal(absolutePath);
                    return expect(server.start()).to.eventually.be.fulfilled;
                });
                it('should start correctly with log, relative paths', () => {
                    const logPath = path.join(relativePath, 'log.txt');
                    server = server_1.default({ log: logPath });
                    expect(server.options.log).to.equal(path.resolve(logPath));
                    return expect(server.start()).to.eventually.be.fulfilled;
                });
                it('should start correctly with custom ssl cert/key, relative paths', () => {
                    server = server_1.default({
                        sslcert: relativeSSLCertPath,
                        sslkey: relativeSSLKeyPath,
                    });
                    expect(server.options.sslcert).to.equal(absoluteSSLCertPath);
                    expect(server.options.sslkey).to.equal(absoluteSSLKeyPath);
                    return expect(server.start()).to.eventually.be.fulfilled;
                });
            });
            context('File Write Mode', () => {
                it("should start correctly with 'overwrite'", () => expect(server_1.default({
                    pactFileWriteMode: 'overwrite',
                }).start()).to.eventually.be.fulfilled);
                it("should start correctly with 'merge'", () => expect(server_1.default({
                    pactFileWriteMode: 'merge',
                }).start()).to.eventually.be.fulfilled);
                it("should start correctly with 'update'", () => expect(server_1.default({
                    pactFileWriteMode: 'update',
                }).start()).to.eventually.be.fulfilled);
            });
            context('Log Level', () => {
                it("should start correctly with 'debug'", () => {
                    return q.allSettled([
                        expect(server_1.default({
                            logLevel: 'debug',
                        }).start()).to.eventually.be.fulfilled,
                        expect(server_1.default({
                            logLevel: 'DEBUG',
                        }).start()).to.eventually.be.fulfilled,
                    ]);
                });
                it("should start correctly with 'info'", () => {
                    return q.allSettled([
                        expect(server_1.default({
                            logLevel: 'info',
                        }).start()).to.eventually.be.fulfilled,
                        expect(server_1.default({
                            logLevel: 'INFO',
                        }).start()).to.eventually.be.fulfilled,
                    ]);
                });
                it("should start correctly with 'warn'", () => {
                    return q.allSettled([
                        expect(server_1.default({
                            logLevel: 'warn',
                        }).start()).to.eventually.be.fulfilled,
                        expect(server_1.default({
                            logLevel: 'WARN',
                        }).start()).to.eventually.be.fulfilled,
                    ]);
                });
                it("should start correctly with 'error'", () => {
                    return q.allSettled([
                        expect(server_1.default({
                            logLevel: 'error',
                        }).start()).to.eventually.be.fulfilled,
                        expect(server_1.default({
                            logLevel: 'ERROR',
                        }).start()).to.eventually.be.fulfilled,
                    ]);
                });
            });
        });
        it('should dispatch event when starting', done => {
            server = server_1.default();
            server.once('start', () => done());
            server.start();
        });
        it('should change running state to true', () => {
            server = server_1.default();
            return server.start().then(() => expect(server['__running']).to.be.true);
        });
    });
    describe('Stop server', () => {
        context('when already started', () => {
            it('should stop running', () => {
                server = server_1.default();
                return server.start().then(() => server.stop());
            });
            it('should dispatch event when stopping', done => {
                server = server_1.default();
                server.once('stop', () => done());
                server.start().then(() => server.stop());
            });
            it('should change running state to false', () => {
                server = server_1.default();
                return server
                    .start()
                    .then(() => server.stop())
                    .then(() => expect(server['__running']).to.be.false);
            });
        });
    });
    describe('Delete server', () => {
        context('when already running', () => {
            it('should stop & delete server', () => {
                server = server_1.default();
                return server.start().then(() => server.delete());
            });
            it('should dispatch event when deleting', done => {
                server = server_1.default();
                server.once('delete', () => done());
                server.start().then(() => server.delete());
            });
            it('should change running state to false', () => {
                server = server_1.default();
                return server
                    .start()
                    .then(() => server.delete())
                    .then(() => expect(server['__running']).to.be.false);
            });
        });
    });
});
//# sourceMappingURL=server.spec.js.map