"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs");
const events = require("events");
const http = require("request");
const q = require("q");
const logger_1 = require("./logger");
const spawn_1 = require("./spawn");
const mkdirp = require("mkdirp");
const checkTypes = require('check-types');
const setTimeout = global.setTimeout;
const CHECKTIME = 500;
const RETRY_AMOUNT = 60;
const PROCESS_TIMEOUT = 30000;
class AbstractService extends events.EventEmitter {
    static get Events() {
        return {
            START_EVENT: 'start',
            STOP_EVENT: 'stop',
            DELETE_EVENT: 'delete',
        };
    }
    constructor(command, options, argMapping, cliVerb) {
        super();
        if (options.logLevel) {
            logger_1.setLogLevel(options.logLevel);
            if (options.logLevel === 'fatal') {
                options.logLevel = 'error';
            }
            else if (options.logLevel === 'trace') {
                options.logLevel = 'debug';
            }
            options.logLevel = options.logLevel.toUpperCase();
        }
        options.ssl = options.ssl || false;
        options.cors = options.cors || false;
        options.host = options.host || 'localhost';
        if (options.port) {
            checkTypes.assert.number(options.port);
            checkTypes.assert.integer(options.port);
            checkTypes.assert.positive(options.port);
            checkTypes.assert.inRange(options.port, 0, 65535);
            if (checkTypes.not.inRange(options.port, 1024, 49151)) {
                logger_1.default.warn('Like a Boss, you used a port outside of the recommended range (1024 to 49151); I too like to live dangerously.');
            }
        }
        checkTypes.assert.boolean(options.ssl);
        if ((options.sslcert && !options.sslkey) ||
            (!options.sslcert && options.sslkey)) {
            throw new Error('Custom ssl certificate and key must be specified together.');
        }
        if (options.sslcert) {
            try {
                fs.statSync(path.normalize(options.sslcert)).isFile();
            }
            catch (e) {
                throw new Error(`Custom ssl certificate not found at path: ${options.sslcert}`);
            }
        }
        if (options.sslkey) {
            try {
                fs.statSync(path.normalize(options.sslkey)).isFile();
            }
            catch (e) {
                throw new Error(`Custom ssl key not found at path: ${options.sslkey}`);
            }
        }
        if (options.sslcert && options.sslkey) {
            options.ssl = true;
        }
        checkTypes.assert.boolean(options.cors);
        if (options.log) {
            const fileObj = path.parse(path.normalize(options.log));
            try {
                fs.statSync(fileObj.dir).isDirectory();
            }
            catch (e) {
                mkdirp.sync(fileObj.dir);
            }
        }
        if (options.host) {
            checkTypes.assert.string(options.host);
        }
        this.options = options;
        this.__running = false;
        this.__cliVerb = cliVerb;
        this.__serviceCommand = command;
        this.__argMapping = argMapping;
    }
    start() {
        if (this.__instance && this.__instance.connected) {
            logger_1.default.warn(`You already have a process running with PID: ${this.__instance.pid}`);
            return q.resolve(this);
        }
        this.__instance = this.spawnBinary();
        this.__instance.once('close', () => this.stop());
        if (!this.options.port) {
            const catchPort = (data) => {
                const match = data.match(/port=([0-9]+)/);
                if (match && match[1]) {
                    this.options.port = parseInt(match[1], 10);
                    if (this.__instance) {
                        this.__instance.stdout.removeListener('data', catchPort);
                    }
                    logger_1.default.info(`Pact running on port ${this.options.port}`);
                }
            };
            this.__instance.stdout.on('data', catchPort);
        }
        this.__instance.stderr.on('data', data => logger_1.default.error(`Pact Binary Error: ${data}`));
        return this.__waitForServiceUp()
            .timeout(PROCESS_TIMEOUT, `Couldn't start Pact with PID: ${this.__instance.pid}`)
            .then(() => {
            this.__running = true;
            this.emit(AbstractService.Events.START_EVENT, this);
            return this;
        });
    }
    stop() {
        const pid = this.__instance ? this.__instance.pid : -1;
        return q(spawn_1.default.killBinary(this.__instance))
            .then(() => this.__waitForServiceDown())
            .timeout(PROCESS_TIMEOUT, `Couldn't stop Pact with PID '${pid}'`)
            .then(() => {
            this.__running = false;
            this.emit(AbstractService.Events.STOP_EVENT, this);
            return this;
        });
    }
    delete() {
        return this.stop().then(() => {
            this.emit(AbstractService.Events.DELETE_EVENT, this);
            return this;
        });
    }
    spawnBinary() {
        return spawn_1.default.spawnBinary(this.__serviceCommand, this.__cliVerb ? [this.__cliVerb, this.options] : [this.options], this.__argMapping);
    }
    __waitForServiceUp() {
        let amount = 0;
        const deferred = q.defer();
        const retry = () => {
            if (amount >= RETRY_AMOUNT) {
                deferred.reject(new Error('Pact startup failed; tried calling service 10 times with no result.'));
            }
            setTimeout(check.bind(this), CHECKTIME);
        };
        const check = () => {
            amount++;
            if (this.options.port) {
                this.__call(this.options).then(() => deferred.resolve(), retry.bind(this));
            }
            else {
                retry();
            }
        };
        check();
        return deferred.promise;
    }
    __waitForServiceDown() {
        let amount = 0;
        const deferred = q.defer();
        const check = () => {
            amount++;
            if (this.options.port) {
                this.__call(this.options).then(() => {
                    if (amount >= RETRY_AMOUNT) {
                        deferred.reject(new Error('Pact stop failed; tried calling service 10 times with no result.'));
                        return;
                    }
                    setTimeout(check, CHECKTIME);
                }, () => deferred.resolve());
            }
            else {
                deferred.resolve();
            }
        };
        check();
        return deferred.promise;
    }
    __call(options) {
        const deferred = q.defer();
        const config = {
            uri: `http${options.ssl ? 's' : ''}://${options.host}:${options.port}`,
            method: 'GET',
            headers: {
                'X-Pact-Mock-Service': true,
                'Content-Type': 'application/json',
            },
        };
        if (options.ssl) {
            process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0';
            config.agentOptions = {};
            config.agentOptions.rejectUnauthorized = false;
            config.agentOptions.requestCert = false;
            config.agentOptions.agent = false;
        }
        http(config, (err, res) => {
            !err && res.statusCode === 200
                ? deferred.resolve()
                : deferred.reject(`HTTP Error: '${JSON.stringify(err ? err : res)}'`);
        });
        return deferred.promise;
    }
}
exports.AbstractService = AbstractService;
//# sourceMappingURL=service.js.map