"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Request = require("request");
const unzipper = require("unzipper");
const tar = require("tar");
const pact_environment_1 = require("../src/pact-environment");
const chalk_1 = require("chalk");
const path = require("path");
const fs = require("fs");
const urljoin = require("url-join");
const config = require('libnpmconfig');
const sumchecker = require('sumchecker');
const request = Request.defaults({
    proxy: process.env.npm_config_https_proxy ||
        process.env.npm_config_proxy ||
        undefined,
});
exports.PACT_STANDALONE_VERSION = '1.88.36';
const PACT_DEFAULT_LOCATION = `https://github.com/pact-foundation/pact-ruby-standalone/releases/download/v${exports.PACT_STANDALONE_VERSION}/`;
const HTTP_REGEX = /^http(s?):\/\//;
function join(...paths) {
    return HTTP_REGEX.test(paths[0]) ? urljoin(...paths) : path.join(...paths);
}
function throwError(msg) {
    throw new Error(chalk_1.default.red(`Error while installing binary: ${msg}`));
}
function getBinaryLocation(location, basePath) {
    if (!location || location.length === 0) {
        return undefined;
    }
    return HTTP_REGEX.test(location)
        ? location
        : path.resolve(basePath, location);
}
function findPackageConfig(location, tries = 10) {
    if (tries === 0) {
        return {};
    }
    const packagePath = path.resolve(location, 'package.json');
    if (fs.existsSync(packagePath)) {
        const config = require(packagePath).config;
        if (config && (config.pact_binary_location || config.pact_do_not_track)) {
            return {
                binaryLocation: getBinaryLocation(config.pact_binary_location, location),
                doNotTrack: config.pact_do_not_track,
            };
        }
    }
    return findPackageConfig(path.resolve(location, '..'), tries - 1);
}
function createConfig() {
    const packageConfig = findPackageConfig(path.resolve(__dirname, '..', '..'));
    const PACT_BINARY_LOCATION = packageConfig.binaryLocation || PACT_DEFAULT_LOCATION;
    const CHECKSUM_SUFFIX = '.checksum';
    return {
        doNotTrack: packageConfig.doNotTrack ||
            process.env.PACT_DO_NOT_TRACK !== undefined ||
            false,
        binaries: [
            {
                platform: 'win32',
                binary: `pact-${exports.PACT_STANDALONE_VERSION}-win32.zip`,
                binaryChecksum: `pact-${exports.PACT_STANDALONE_VERSION}-win32.zip${CHECKSUM_SUFFIX}`,
                downloadLocation: PACT_BINARY_LOCATION,
                folderName: `win32-${exports.PACT_STANDALONE_VERSION}`,
            },
            {
                platform: 'darwin',
                binary: `pact-${exports.PACT_STANDALONE_VERSION}-osx.tar.gz`,
                binaryChecksum: `pact-${exports.PACT_STANDALONE_VERSION}-osx.tar.gz${CHECKSUM_SUFFIX}`,
                downloadLocation: PACT_BINARY_LOCATION,
                folderName: `darwin-${exports.PACT_STANDALONE_VERSION}`,
            },
            {
                platform: 'linux',
                arch: 'x64',
                binary: `pact-${exports.PACT_STANDALONE_VERSION}-linux-x86_64.tar.gz`,
                binaryChecksum: `pact-${exports.PACT_STANDALONE_VERSION}-linux-x86_64.tar.gz${CHECKSUM_SUFFIX}`,
                downloadLocation: PACT_BINARY_LOCATION,
                folderName: `linux-x64-${exports.PACT_STANDALONE_VERSION}`,
            },
            {
                platform: 'linux',
                arch: 'ia32',
                binary: `pact-${exports.PACT_STANDALONE_VERSION}-linux-x86.tar.gz`,
                binaryChecksum: `pact-${exports.PACT_STANDALONE_VERSION}-linux-x86.tar.gz${CHECKSUM_SUFFIX}`,
                downloadLocation: PACT_BINARY_LOCATION,
                folderName: `linux-ia32-${exports.PACT_STANDALONE_VERSION}`,
            },
        ],
    };
}
exports.createConfig = createConfig;
const CONFIG = createConfig();
const CIs = [
    'CI',
    'CONTINUOUS_INTEGRATION',
    'ABSTRUSE_BUILD_DIR',
    'APPVEYOR',
    'BUDDY_WORKSPACE_URL',
    'BUILDKITE',
    'CF_BUILD_URL',
    'CIRCLECI',
    'CODEBUILD_BUILD_ARN',
    'CONCOURSE_URL',
    'DRONE',
    'GITLAB_CI',
    'GO_SERVER_URL',
    'JENKINS_URL',
    'PROBO_ENVIRONMENT',
    'SEMAPHORE',
    'SHIPPABLE',
    'TDDIUM',
    'TEAMCITY_VERSION',
    'TF_BUILD',
    'TRAVIS',
    'WERCKER_ROOT',
];
function downloadFileRetry(url, filepath, retry = 3) {
    return new Promise((resolve, reject) => {
        let len = 0;
        let downloaded = 0;
        let time = Date.now();
        let ca = config.read()['cafile'];
        if (ca) {
            ca = fs.readFileSync(ca);
        }
        request({
            url,
            headers: {
                'User-Agent': 'https://github.com/pact-foundation/pact-node',
            },
            strictSSL: config.read()['strict-ssl'],
            agentOptions: {
                ca: ca,
            },
        })
            .on('error', (e) => reject(e))
            .on('response', (res) => (len = parseInt(res.headers['content-length'], 10)))
            .on('data', (chunk) => {
            downloaded += chunk.length;
            const now = Date.now();
            if (now - time > 1000) {
                time = now;
                console.log(chalk_1.default.gray(`Downloaded ${((100 * downloaded) / len).toFixed(2)}%...`));
            }
        })
            .pipe(fs.createWriteStream(filepath))
            .on('finish', () => resolve());
    }).catch((e) => retry-- === 0 ? throwError(e) : downloadFileRetry(url, filepath, retry));
}
function download(data) {
    console.log(chalk_1.default.gray(`Installing Pact Standalone Binary for ${data.platform}.`));
    return new Promise((resolve, reject) => {
        if (fs.existsSync(path.resolve(data.filepath))) {
            console.log(chalk_1.default.yellow('Binary already downloaded, skipping...'));
            resolve(data);
            return;
        }
        console.log(chalk_1.default.yellow(`Downloading Pact Standalone Binary v${exports.PACT_STANDALONE_VERSION} for platform ${data.platform} from ${data.binaryDownloadPath}`));
        if (!CONFIG.doNotTrack) {
            console.log(chalk_1.default.gray('Please note: we are tracking this download anonymously to gather important usage statistics. ' +
                "To disable tracking, set 'pact_do_not_track: true' in your package.json 'config' section."));
            const isCI = CIs.some(key => process.env[key] !== undefined);
            request
                .post({
                url: 'https://www.google-analytics.com/collect',
                form: {
                    v: 1,
                    tid: 'UA-117778936-1',
                    cid: Math.round(2147483647 * Math.random()).toString(),
                    t: 'screenview',
                    an: 'pact-install',
                    av: require('../package.json').version,
                    aid: 'pact-node',
                    aiid: `standalone-${exports.PACT_STANDALONE_VERSION}`,
                    cd: `download-node-${data.platform}-${isCI ? 'ci' : 'user'}`,
                    aip: true,
                },
            })
                .on('error', () => {
            });
        }
        if (HTTP_REGEX.test(data.binaryDownloadPath)) {
            downloadFileRetry(data.binaryDownloadPath, data.filepath).then(() => {
                console.log(chalk_1.default.green(`Finished downloading binary to ${data.filepath}`));
                resolve(data);
            }, (e) => reject(`Error downloading binary from ${data.binaryDownloadPath}: ${e}`));
        }
        else if (fs.existsSync(data.binaryDownloadPath)) {
            fs.createReadStream(data.binaryDownloadPath)
                .on('error', (e) => reject(`Error reading the file at '${data.binaryDownloadPath}': ${e}`))
                .pipe(fs
                .createWriteStream(data.filepath)
                .on('error', (e) => reject(`Error writing the file to '${data.filepath}': ${e}`))
                .on('close', () => resolve(data)));
        }
        else {
            reject(`Could not get binary from '${data.binaryDownloadPath}' as it's not a URL and does not exist at the path specified.`);
        }
    });
}
function extract(data) {
    if (fs.existsSync(data.platformFolderPath)) {
        return Promise.resolve();
    }
    if (!fs.existsSync(data.checksumFilepath)) {
        throwError(`Checksum file missing from standalone directory. Aborting.`);
    }
    fs.mkdirSync(data.platformFolderPath);
    console.log(chalk_1.default.yellow(`Extracting binary from ${data.filepath}.`));
    const basename = path.basename(data.filepath);
    return (sumchecker('sha1', data.checksumFilepath, __dirname, basename)
        .then(() => console.log(chalk_1.default.green(`Checksum passed for '${basename}'.`)), () => throwError(`Checksum rejected for file '${basename}' with checksum ${path.basename(data.checksumFilepath)}`))
        .then(() => data.isWindows
        ? fs
            .createReadStream(data.filepath)
            .pipe(unzipper.Extract({
            path: data.platformFolderPath,
        }))
            .on('entry', entry => entry.autodrain())
            .promise()
        : tar.x({
            file: data.filepath,
            cwd: data.platformFolderPath,
            preserveOwner: false,
        }))
        .then(() => {
        const publishPath = path.resolve(data.platformFolderPath, 'pact', 'bin', `pact-publish${pact_environment_1.default.isWindows() ? '.bat' : ''}`);
        if (fs.existsSync(publishPath)) {
            fs.unlinkSync(publishPath);
        }
        console.log(chalk_1.default.green('Extraction done.'));
    })
        .then(() => {
        console.log('\n\n' +
            chalk_1.default.bgYellow(chalk_1.default.black('### If you') +
                chalk_1.default.red(' ❤ ') +
                chalk_1.default.black('Pact and want to support us, please donate here:')) +
            chalk_1.default.blue(' http://donate.pact.io/node') +
            '\n\n');
    })
        .catch((e) => throwError(`Extraction failed for ${data.filepath}: ${e}`)));
}
function getBinaryEntry(platform, arch) {
    platform = platform || process.platform;
    arch = arch || process.arch;
    for (let value of CONFIG.binaries) {
        if (value.platform === platform &&
            (value.arch ? value.arch === arch : true)) {
            return value;
        }
    }
    throw throwError(`Cannot find binary for platform '${platform}' with architecture '${arch}'.`);
}
exports.getBinaryEntry = getBinaryEntry;
function setup(platform, arch) {
    const entry = getBinaryEntry(platform, arch);
    return Promise.resolve({
        binaryDownloadPath: join(entry.downloadLocation, entry.binary),
        checksumDownloadPath: join(PACT_DEFAULT_LOCATION, entry.binaryChecksum),
        filepath: path.resolve(__dirname, entry.binary),
        checksumFilepath: path.resolve(__dirname, entry.binaryChecksum),
        isWindows: pact_environment_1.default.isWindows(platform),
        platform: entry.platform,
        arch: entry.arch,
        platformFolderPath: path.resolve(__dirname, entry.folderName),
    });
}
function downloadChecksums() {
    console.log(chalk_1.default.gray(`Downloading All Pact Standalone Binary Checksums.`));
    return Promise.all(CONFIG.binaries.map(value => setup(value.platform, value.arch).then((data) => downloadFileRetry(data.checksumDownloadPath, data.checksumFilepath).then(() => {
        console.log(chalk_1.default.green(`Finished downloading checksum ${path.basename(data.checksumFilepath)}`));
        return data;
    }, (e) => throwError(`Error downloading checksum from ${data.checksumDownloadPath}: ${e}`))))).then(() => console.log(chalk_1.default.green('All checksums downloaded.')), (e) => throwError(`Checksum Download Failed Unexpectedly: ${e}`));
}
exports.downloadChecksums = downloadChecksums;
exports.default = (platform, arch) => {
    if (process.env.PACT_SKIP_BINARY_INSTALL) {
        console.log(chalk_1.default.yellow("Skipping binary installation. Env var 'PACT_SKIP_BINARY_INSTALL' was found."));
        return Promise.resolve();
    }
    return setup(platform, arch)
        .then(d => download(d))
        .then(d => extract(d))
        .then(() => console.log(chalk_1.default.green('Pact Standalone Binary is ready.')))
        .catch((e) => throwError(`Postinstalled Failed Unexpectedly: ${e}`));
};
//# sourceMappingURL=install.js.map