"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const chai = require("chai");
const install_1 = require("./install");
const expect = chai.expect;
describe('Install', () => {
    const packageBasePath = path.resolve(__dirname, '../..');
    const packagePath = path.resolve(packageBasePath, 'package.json');
    beforeEach(() => {
        for (let key in require.cache) {
            delete require.cache[key];
        }
    });
    function createConfig() {
        return require('./install').createConfig();
    }
    describe('Package.json Configuration', () => {
        let packageConfig;
        beforeEach(() => (packageConfig = JSON.parse(JSON.stringify(require('../package.json')))));
        afterEach(() => {
            if (fs.existsSync(packagePath)) {
                fs.unlinkSync(packagePath);
            }
        });
        describe('Binary Location', () => {
            function setBinaryLocation(location, expectation) {
                packageConfig.config = {
                    pact_binary_location: location,
                };
                fs.writeFileSync(packagePath, JSON.stringify(packageConfig));
                const config = createConfig();
                config.binaries.forEach((entry) => {
                    expect(entry.downloadLocation).to.be.equal(expectation || location);
                });
            }
            it('Should be able to set binary location as an absolute path', () => setBinaryLocation('/some-location/or-other', path.resolve('/some-location/or-other')));
            it('Should be able to set binary location as an relative path', () => {
                const location = 'some-location/or-other';
                setBinaryLocation(location, path.resolve(packageBasePath, location));
            });
            it('Should be able to set binary location as an HTTP URL', () => setBinaryLocation('http://some.url'));
            it('Should be able to set binary location as an HTTPS URL', () => setBinaryLocation('https://some.url'));
        });
        it("Should be able to set 'do not track' from package.json config", () => {
            const doNotTrack = true;
            packageConfig.config = {
                pact_do_not_track: doNotTrack,
            };
            fs.writeFileSync(packagePath, JSON.stringify(packageConfig));
            const config = createConfig();
            expect(config.doNotTrack).to.be.equal(doNotTrack);
        });
    });
    describe('Environment variables configuration', () => {
        it("Should be able to set 'do not track' from environment variable 'PACT_DO_NOT_TRACK'", () => {
            const doNotTrack = true;
            process.env.PACT_DO_NOT_TRACK = `${doNotTrack}`;
            const config = createConfig();
            expect(config.doNotTrack).to.be.equal(doNotTrack);
        });
    });
    describe('When skips install', () => {
        let OLD_ENV = Object.assign({}, process.env);
        beforeEach(() => {
            process.env.PACT_SKIP_BINARY_INSTALL = 'true';
        });
        afterEach(() => {
            process.env = Object.assign({}, OLD_ENV);
        });
        it('Should not download it', () => {
            expect(install_1.default('Linux', 'x86')).to.be.fulfilled;
        });
    });
});
//# sourceMappingURL=install.spec.js.map