"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const chai = require("chai");
const logger_1 = require("../src/logger");
const chaiAsPromised = require("chai-as-promised");
const publisher_1 = require("../src/publisher");
const broker_mock_1 = require("./integration/broker-mock");
const expect = chai.expect;
chai.use(chaiAsPromised);
describe('Publish Spec', () => {
    let server;
    const PORT = Math.floor(Math.random() * 999) + 9000;
    const pactBrokerBaseUrl = `http://localhost:${PORT}`;
    const authenticatedPactBrokerBaseUrl = `${pactBrokerBaseUrl}/auth`;
    before(() => broker_mock_1.default(PORT).then(s => {
        logger_1.default.debug(`Pact Broker Mock listening on port: ${PORT}`);
        server = s;
    }));
    after(() => server.close());
    context('when publishing a to a broker', () => {
        context('without authentication', () => {
            context('and the Pact contract is valid', () => {
                it('should successfully publish all Pacts', () => {
                    const publisher = publisher_1.default({
                        pactBroker: pactBrokerBaseUrl,
                        pactFilesOrDirs: [
                            path.resolve(__dirname, 'integration/publish/publish-success.json'),
                        ],
                        consumerVersion: '1.0.0',
                    });
                    expect(publisher).to.be.a('object');
                    expect(publisher).to.respondTo('publish');
                    return expect(publisher.publish()).to.eventually.be.fulfilled;
                });
                it('should successfully tag all Pacts with `test` and `latest`', () => {
                    const publisher = publisher_1.default({
                        pactBroker: pactBrokerBaseUrl,
                        pactFilesOrDirs: [
                            path.resolve(__dirname, 'integration/publish/publish-success.json'),
                        ],
                        consumerVersion: '1.0.0',
                        tags: ['test', 'latest'],
                    });
                    expect(publisher).to.be.a('object');
                    expect(publisher).to.respondTo('publish');
                    return expect(publisher.publish()).to.eventually.be.fulfilled;
                });
            });
            context('and the Pact contract is invalid', () => {
                it('should report an unsuccessful push', () => {
                    const publisher = publisher_1.default({
                        pactBroker: pactBrokerBaseUrl,
                        pactFilesOrDirs: [
                            path.resolve(__dirname, 'integration/publish/publish-fail.json'),
                        ],
                        consumerVersion: '1.0.0',
                    });
                    expect(publisher).to.be.a('object');
                    expect(publisher).to.respondTo('publish');
                    return expect(publisher.publish()).to.eventually.be.rejected;
                });
            });
        });
        context('with authentication', () => {
            context('and valid credentials', () => {
                it('should return a sucessful promise', () => {
                    const publisher = publisher_1.default({
                        pactBroker: authenticatedPactBrokerBaseUrl,
                        pactFilesOrDirs: [
                            path.resolve(__dirname, 'integration/publish/publish-success.json'),
                        ],
                        consumerVersion: '1.0.0',
                        pactBrokerUsername: 'foo',
                        pactBrokerPassword: 'bar',
                    });
                    expect(publisher).to.be.a('object');
                    expect(publisher).to.respondTo('publish');
                    return expect(publisher.publish()).to.eventually.be.fulfilled;
                });
            });
            context('and invalid credentials', () => {
                it('should return a rejected promise', () => {
                    const publisher = publisher_1.default({
                        pactBroker: authenticatedPactBrokerBaseUrl,
                        pactFilesOrDirs: [
                            path.resolve(__dirname, 'integration/publish/publish-success.json'),
                        ],
                        consumerVersion: '1.0.0',
                        pactBrokerUsername: 'not',
                        pactBrokerPassword: 'right',
                    });
                    expect(publisher).to.be.a('object');
                    expect(publisher).to.respondTo('publish');
                    return expect(publisher.publish()).to.eventually.be.rejected;
                });
            });
        });
    });
    context('when publishing a directory of Pacts to a Broker', () => {
        context('and the directory contains only valid Pact contracts', () => {
            it('should asynchronously send all Pacts to the Broker', () => {
                const publisher = publisher_1.default({
                    pactBroker: pactBrokerBaseUrl,
                    pactFilesOrDirs: [
                        path.resolve(__dirname, 'integration/publish/pactDirTests'),
                    ],
                    consumerVersion: '1.0.0',
                });
                expect(publisher).to.be.a('object');
                expect(publisher).to.respondTo('publish');
                return expect(publisher.publish()).to.eventually.be.fulfilled;
            });
            it('should successfully tag all Pacts sent with `test` and `latest`', () => {
                const publisher = publisher_1.default({
                    pactBroker: pactBrokerBaseUrl,
                    pactFilesOrDirs: [
                        path.resolve(__dirname, 'integration/publish/pactDirTests'),
                    ],
                    consumerVersion: '1.0.0',
                    tags: ['test', 'latest'],
                });
                expect(publisher).to.be.a('object');
                expect(publisher).to.respondTo('publish');
                return expect(publisher.publish()).to.eventually.be.fulfilled;
            });
        });
        context('and the directory contains Pact and non-Pact contracts', () => {
            it('should asynchronously send only the Pact contracts to the broker', () => {
                const publisher = publisher_1.default({
                    pactBroker: pactBrokerBaseUrl,
                    pactFilesOrDirs: [
                        path.resolve(__dirname, 'integration/publish/pactDirTestsWithOtherStuff'),
                    ],
                    consumerVersion: '1.0.0',
                });
                expect(publisher).to.be.a('object');
                expect(publisher).to.respondTo('publish');
                return expect(publisher.publish()).to.eventually.be.fulfilled;
            });
        });
    });
});
//# sourceMappingURL=publisher.integration.spec.js.map